@@ -4,8 +4,8 @@
 #  --> https://CRAN.R-project.org/package=spam              #
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
-# by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Roman Flury [ctb], Daniel Gerber [ctb],                #
+# by Reinhard Furrer [aut, cre], Florian Gerber [aut],      #
+#    Roman Flury [aut], Daniel Gerber [ctb],                #
 #    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 rm(list = ls())
@@ -67,6 +67,8 @@ test_that("option 'chol.update'", {
     ss1 <- ss+diag.spam(dim(ss)[1])
     spamtest_eq( chol(ss), update.spam.chol.NgPeyton(css, ss))
     
+    spamtest_eq( chol(ss, Rstruct=css), update.spam.chol.NgPeyton(css, ss))
+    
     sel <- which(ss[1,,drop=TRUE]!=0)
     ss1[1,sel[-1]] <- 0
     ss2 <- ss
